﻿Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.EditorInput
Imports Autodesk.AutoCAD.Geometry

Public Class Class1
    <CommandMethod("DL_VBNET")>
    Public Sub DrawLine()
        '' Get the current document and database
        Dim acDoc As Document = Application.DocumentManager.MdiActiveDocument
        Dim acCurDb As Database = acDoc.Database

        '' Start a transaction
        Using acTrans As Transaction = acCurDb.TransactionManager.StartTransaction()

            '' Open the Block table for read
            Dim acBlkTbl As BlockTable
            acBlkTbl = acTrans.GetObject(acCurDb.BlockTableId, OpenMode.ForRead)

            '' Open the Block table record Model space for write
            Dim acBlkTblRec As BlockTableRecord
            acBlkTblRec = acTrans.GetObject(acBlkTbl(BlockTableRecord.ModelSpace),
                                            OpenMode.ForWrite)

            '' Create a line that starts at 0,0 and ends at 150,150
            Dim acLine As Line = New Line(New Point3d(0, 0, 0),
                                          New Point3d(150, 150, 0))

            '' Add the new object to the block table record and the transaction
            acBlkTblRec.AppendEntity(acLine)
            acTrans.AddNewlyCreatedDBObject(acLine, True)

            '' Save the new object to the database
            acTrans.Commit()
        End Using
    End Sub

    <CommandMethod("MYTEXT_VBNET")>
    Public Sub MyText()
        ' Get the current document and database
        Dim acDoc As Document = Application.DocumentManager.MdiActiveDocument
        Dim acCurDb As Database = acDoc.Database

        ' Prompt for the start point
        Dim pPt1Res As PromptPointResult
        Dim pPt1Opts As PromptPointOptions = New PromptPointOptions("")
        pPt1Opts.Message = vbLf & "Specify the first point of the rectangle: "
        pPt1Res = acDoc.Editor.GetPoint(pPt1Opts)

        ' Prompt for the second point
        If pPt1Res.Status = PromptStatus.OK Then
            Dim pPt2Res As PromptPointResult
            Dim pPt2Opts As PromptPointOptions = New PromptPointOptions("")
            pPt2Res = acDoc.Editor.GetCorner(vbLf & "Specify the opposite corner of the rectangle: ", pPt1Res.Value)

            ' Proceed if a second point was specified
            If pPt2Res.Status = PromptStatus.OK Then
                ' Set our layer and text style current
                Application.SetSystemVariable("CLAYER", "TS-TEXT2")
                Application.SetSystemVariable("TEXTSTYLE", "SANSSERIFBOLD")

                ' Start the MText command for interactivity with the editor
                acDoc.SendStringToExecute("._MTEXT " + pPt1Res.Value.X.ToString + "," + pPt1Res.Value.Y.ToString + " " +
                                          "_H 0.75 " + pPt2Res.Value.X.ToString + "," + pPt2Res.Value.Y.ToString + " ", True, False, True)
            End If
        End If
    End Sub

    <CommandMethod("MYTEXT_IPE_VBNET")>
    Public Sub MyText_INPlaceEditor()
        ' Get the current document and database
        Dim acDoc As Document = Application.DocumentManager.MdiActiveDocument
        Dim acCurDb As Database = acDoc.Database

        ' Prompt for the start point
        Dim pPt1Res As PromptPointResult
        Dim pPt1Opts As PromptPointOptions = New PromptPointOptions("")
        pPt1Opts.Message = vbLf & "Specify the first point of the rectangle: "
        pPt1Res = acDoc.Editor.GetPoint(pPt1Opts)

        ' Prompt for the second point
        If pPt1Res.Status = PromptStatus.OK Then
            Dim pPt2Res As PromptPointResult
            Dim pPt2Opts As PromptPointOptions = New PromptPointOptions("")
            pPt2Res = acDoc.Editor.GetCorner(vbLf & "Specify the opposite corner of the rectangle: ", pPt1Res.Value)

            ' Proceed if a second point was specified
            If pPt2Res.Status = PromptStatus.OK Then
                ' Set our layer and text style current
                Application.SetSystemVariable("CLAYER", "TS-TEXT2")
                Application.SetSystemVariable("TEXTSTYLE", "SANSSERIFBOLD")

                ' Start a transaction
                Using acTrans As Transaction = acCurDb.TransactionManager.StartTransaction()
                    ' Create a new MText object
                    Dim newMText As MText = New MText()
                    newMText.Attachment = AttachmentPoint.TopLeft
                    newMText.Location = pPt1Res.Value
                    newMText.Height = 0.75
                    newMText.Width = Math.Abs(pPt1Res.Value.X - pPt2Res.Value.X)

                    ' Display the MText in-place editor
                    InplaceTextEditor.Invoke(newMText, New InplaceTextEditorSettings())

                    ' Open the Block table for read
                    Dim acBlkTbl As BlockTable
                    acBlkTbl = acTrans.GetObject(acCurDb.BlockTableId, OpenMode.ForRead)

                    ' Open the Block table record Model space for write
                    Dim acBlkTblRec As BlockTableRecord
                    acBlkTblRec = acTrans.GetObject(acBlkTbl(BlockTableRecord.ModelSpace),
                                            OpenMode.ForWrite)

                    ' Add the new object to the block table record and the transaction
                    acBlkTblRec.AppendEntity(newMText)
                    acTrans.AddNewlyCreatedDBObject(newMText, True)

                    ' Save the new object to the database
                    acTrans.Commit()
                End Using
            End If
        End If
    End Sub
End Class
