// Load the common Windows headers
#include <windows.h>

// Load the common AutoCAD headers
#include "arxHeaders.h"
#include "acedCmdNF.h"
#include "dbents.h"

static void Drawline();
static void MyText();

extern "C" AcRx::AppRetCode acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt)
{
    switch(msg)
    {
        case AcRx::kInitAppMsg:
            acrxDynamicLinker->unlockApplication(pkt);
            acrxDynamicLinker->registerAppMDIAware(pkt);

            // Commands to add
            acedRegCmds->addCommand(L"Samples_ARX", L"DL_ObjectARX", 
                                    L"DL_ObjectARX", ACRX_CMD_MODAL, Drawline);
            acedRegCmds->addCommand(L"Samples_ARX", L"MYTEXT_ObjectARX",
                                    L"MYTEXT_ObjectARX", ACRX_CMD_MODAL, MyText);
            break;
        case AcRx::kUnloadAppMsg:

            // Command Groups to remove
            acedRegCmds->removeGroup(L"Samples_ARX");

            break;
        default:
            break;
    }
    return AcRx::kRetOK;
}

static void Drawline()
{
    // Get the current database
    AcDbDatabase *pDb = acdbHostApplicationServices()->workingDatabase();

    // Open the Block Table for read-only
    AcDbBlockTable *pBlockTable;
    pDb->getSymbolTable(pBlockTable, AcDb::kForRead);

    // Get the Model Space block
    AcDbBlockTableRecord *pBlockTableRecord;
    pBlockTable->getAt(ACDB_MODEL_SPACE, pBlockTableRecord, AcDb::kForWrite);
    pBlockTable->close();

    // Define the points that will be used to define the Line object
    AcGePoint3d startPt(0.0, 0.0, 0.0);
    AcGePoint3d endPt(150.0, 150.0, 0.0);

    // Create the new Line object in memory
    AcDbLine *pLine = new AcDbLine(startPt, endPt);

    // Add the new Line object to Model space
    pBlockTableRecord->appendAcDbEntity(pLine);

    // Close the Model space block
    pBlockTableRecord->close();

    // Close the new line object
    pLine->close();
}

static Adesk::Boolean isMtextActive()
{
    struct resbuf rb;
    acedGetVar(L"CMDNAMES", &rb);
    if (rb.resval.rstring != NULL)
       if (rb.resval.rstring == L"MTEXT")
          return Adesk::kTrue;

    return Adesk::kFalse;
}

static int myMTextCallbackFn(void* pData)
{
    if (isMtextActive())
    {
        acedCommandC(&myMTextCallbackFn, NULL, RTSTR, PAUSE, RTNONE);
        return 1;
    }

    return 0;
}

static void MyText()
{
    // Get the current database
    AcDbDatabase* pDb = acdbHostApplicationServices()->workingDatabase();

    // Open the Layer Table for read-only
    AcDbLayerTable* pLayerTable;
    pDb->getSymbolTable(pLayerTable, AcDb::kForRead);

    // Set the TS-TEXT2 layer current
    AcDbLayerTableRecord* pLayerTableRecord;
    if (pLayerTable->has(L"TS-TEXT2"))
    {
        pLayerTable->getAt(L"TS-TEXT2", pLayerTableRecord, AcDb::kForRead);
        pLayerTable->close();

        pDb->setClayer(pLayerTableRecord->objectId());
        pLayerTableRecord->close();
    }

    // Open the Text Style Table for read-only
    AcDbTextStyleTable* pTextStyleTable;
    pDb->getSymbolTable(pTextStyleTable, AcDb::kForRead);

    // Set the SANSSERIFBOLD text style current
    AcDbTextStyleTableRecord* pTextStyleTableRecord;
    if (pTextStyleTable->has(L"SANSSERIFBOLD"))
    {
        pTextStyleTable->getAt(L"SANSSERIFBOLD", pTextStyleTableRecord, AcDb::kForRead);
        pTextStyleTable->close();

        pDb->setTextstyle(pTextStyleTableRecord->objectId());
        pTextStyleTableRecord->close();
    }

    // Create 2 variables point data type
    ads_point pt1, pt2;

    // Prompt for the first point
    if (RTNORM == acedGetPoint(NULL, L"\nSpecify the first point of the rectangle: ", pt1))
    {
        AcGePoint3d startPt(pt1[0], pt1[1], pt1[2]);

        // Prompt for the second point
        if (RTNORM == acedGetCorner(pt1, L"\nSpecify the opposite corner of the rectangle: ", pt2))
        {
            AcGePoint3d endPt(pt2[0], pt2[1], pt2[2]);

            acedInitDialog();
            acedCommandC(&myMTextCallbackFn, NULL, RTSTR, L"._MTEXT", RT3DPOINT, pt1, RTSTR, L"_H", RTREAL, 0.75, RT3DPOINT, pt2, RTNONE);
        }
    }
}